/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCanvasPixelArray.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/PrivateGdomCanvasPixelArray.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateIntGdomCanvasPixelArray.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::CanvasPixelArray* obj)
{
    return WebCore::cacheDOMObject<WebCore::CanvasPixelArray, GDOMCanvasPixelArray>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMCanvasPixelArray::GDOMCanvasPixelArray(PassRefPtr<WebCore::CanvasPixelArray> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMCanvasPixelArray::~GDOMCanvasPixelArray()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CanvasPixelArray.h"
#include <gdom/GdomCanvasPixelArray.h>
#include <gdom/PrivateGdomCanvasPixelArray.h>
#include <gdom/PrivateIntGdomCanvasPixelArray.h>
#include <gdom/GdomCanvasPixelArrayprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCanvasPixelArray, gdom_canvas_pixel_array, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomCanvasPixelArrayPrivate {
    class WebCore::CanvasPixelArray *item;
};

#define GDOM_CANVAS_PIXEL_ARRAY_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CANVAS_PIXEL_ARRAY, GdomCanvasPixelArrayPrivate))
*/
G_END_DECLS
WebCore::CanvasPixelArray *
gdom_canvas_pixel_array_get_private(GdomCanvasPixelArray *request)
{
    GDOMCanvasPixelArray *res;
    res = static_cast<GDOMCanvasPixelArray*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CanvasPixelArray*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_LENGTH,
};


static void gdom_canvas_pixel_array_finalize(GObject* object)
{
    /*GdomCanvasPixelArray* request = GDOM_CANVAS_PIXEL_ARRAY(object);*/
    /*GdomCanvasPixelArrayPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_canvas_pixel_array_parent_class)->finalize(object);
}

static void gdom_canvas_pixel_array__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCanvasPixelArray *self = GDOM_CANVAS_PIXEL_ARRAY(object);
    WebCore::CanvasPixelArray * gself = (gdom_canvas_pixel_array_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_canvas_pixel_array__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCanvasPixelArray *self = GDOM_CANVAS_PIXEL_ARRAY(object);
    WebCore::CanvasPixelArray * gself = (gdom_canvas_pixel_array_get_private(self));
    switch (prop_id) {
    case PROP_LENGTH:
    {
         g_value_set_long (value, gself->length());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_canvas_pixel_array_class_init(GdomCanvasPixelArrayClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_canvas_pixel_array_finalize;
    g_klass->set_property = gdom_canvas_pixel_array__set_property_fn;
    g_klass->get_property = gdom_canvas_pixel_array__get_property_fn;

    g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_long ("length", /* name */
                "canvas_pixel_array_length", /* short description */
                "read-only  glong CanvasPixelArray.length", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCanvasPixelArrayPrivate));*/
}

static void gdom_canvas_pixel_array_init(GdomCanvasPixelArray* request)
{
    /*GdomCanvasPixelArrayPrivate* priv = GDOM_CANVAS_PIXEL_ARRAY_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

